# encoding: utf-8

import sys

from gi.repository import (
    GLib,
    Gio,
    Unity
)

from scope import MockScope

__rendererMap__ = {
    'vertical_tile': Unity.CategoryRenderer.VERTICAL_TILE
}

__filterMap__ = {
    'radio': Unity.RadioOptionFilter
}

class MockLens(object):
    '''Mock Unity lens'''

    __data = None
    __lens = None

    def __init__(self, data):
        '''Command line options.'''
        self.__data = data
        self.__requestName(self.__data['lens']['dbusname'])
        
        self.__lens = Unity.Lens.new(self.__data['lens']['dbuspath'], self.__data['lens']['id'])
        
        self.__lens.props.search_hint = self.__data['lens']['searchhint']
        self.__lens.props.visible = self.__data['lens']['visible']
        self.__lens.props.search_in_global = self.__data['lens']['global']

        self.__populate_categories()
        self.__populate_filters()

        for scope in self.__data['scopes']:
            s = MockScope(scope)
            self.__lens.add_local_scope(s.getUnityScope())
        
        self.__lens.export()

    def __requestName(self, dbusname):
        '''Connect to the session bus, exit if there is a running instance.'''
        try:
            session_bus_connection = Gio.bus_get_sync(Gio.BusType.SESSION, None)
            session_bus = Gio.DBusProxy.new_sync(session_bus_connection, 0, None,
                                                    'org.freedesktop.DBus',
                                                    '/org/freedesktop/DBus',
                                                    'org.freedesktop.DBus', None)
            result = session_bus.call_sync('RequestName',
                                            GLib.Variant("(su)", (dbusname, 0x4)),
                                            0, -1, None)
    
            # Unpack variant response with signature "(u)". 1 means we got it.
            result = result.unpack()[0]
    
            if result != 1:
                print >> sys.stderr, "Failed to own dbusname %s. Bailing out." % dbusname
                raise SystemExit(1)
        except:
            raise SystemExit(1)
        
    def __populate_categories(self):
        '''Populate categories'''
        categories = []
        for category in self.__data['lens']['categories']:
            categories.append(
                Unity.Category.new(
                    category['name'],
                    Gio.ThemedIcon.new(category['iconpath']),
                    __rendererMap__[category['renderer']]
            ))
        
        self.__lens.props.categories = categories
        
    def __populate_filters(self):
        filters = []
        try:
            for filterData in self.__data['lens']['filters']:
                f = __filterMap__[filterData['type']].new(
                    filterData['id'],
                    filterData['name'],
                    filterData['iconpath'] and Gio.ThemedIcon.new(filterData['iconpath']) or None,
                    filterData['collapsed']
                )
                for optionData in filterData['options']:
                    f.add_option(*optionData)
                filters.append(f)
        except KeyError:
            pass

        self.__lens.props.filters = filters